import ulab.numpy as np
import nncase_runtime as nn
from media.camera import *
from media.display import *
from media.media import *
import image
import time
import gc
import ujson
import aicube

root_path="/sdcard/mp_deployment_source/"        # root_path要以/结尾
config_path=root_path+"deploy_config.json"
image_path=root_path+"test.jpg"
deploy_conf={}
debug_mode=1

class ScopedTiming:
    def __init__(self, info="", enable_profile=True):
        self.info = info
        self.enable_profile = enable_profile

    def __enter__(self):
        if self.enable_profile:
            self.start_time = time.time_ns()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.enable_profile:
            elapsed_time = time.time_ns() - self.start_time
            print(f"{self.info} took {elapsed_time / 1000000:.2f} ms")


def read_img(img_path):
    img_data = image.Image(img_path)
    img_data_rgb888=img_data.to_rgb888()
    img_hwc=img_data_rgb888.to_numpy_ref()
    shape=img_hwc.shape
    img_tmp = img_hwc.reshape((shape[0] * shape[1], shape[2]))
    img_tmp_trans = img_tmp.transpose()
    img_res=img_tmp_trans.copy()
    img_return=img_res.reshape((shape[2],shape[0],shape[1]))
    return img_return

def read_deploy_config(config_path):
    # 打开JSON文件以进行读取deploy_config
    with open(config_path, 'r') as json_file:
        try:
            # 从文件中加载JSON数据
            config = ujson.load(json_file)

            # 打印数据（可根据需要执行其他操作）
            #print(config)
        except ValueError as e:
            print("JSON 解析错误:", e)
    return config

def segmentation():
    print("--------------start-----------------")


    # 使用json读取内容初始化部署变量
    deploy_conf=read_deploy_config(config_path)
    kmodel_name=deploy_conf["kmodel_path"]
    labels=deploy_conf["categories"]
    confidence_threshold=deploy_conf["confidence_threshold"]
    img_size=deploy_conf["img_size"]
    num_classes=deploy_conf["num_classes"]
    cls_idx=-1

    # init kpu and load kmodel
    kpu = nn.kpu()
    kpu.load_kmodel(root_path+kmodel_name)

    # ai2d输入输出初始化
    ai2d_pre_input = read_img(image_path)
    ai2d_pre_input_tensor = nn.from_numpy(ai2d_pre_input)
    ai2d_pre_input_shape=ai2d_pre_input.shape
    data = np.ones((1,3,img_size[0],img_size[1]),dtype=np.uint8)
    ai2d_pre_out = nn.from_numpy(data)

    #---preprocess---
    ai2d_pre = nn.ai2d()
    ai2d_pre.set_dtype(nn.ai2d_format.NCHW_FMT,
                                   nn.ai2d_format.NCHW_FMT,
                                   np.uint8, np.uint8)
    ai2d_pre.set_resize_param(True, nn.interp_method.tf_bilinear, nn.interp_mode.half_pixel )
    ai2d_pre_builder = ai2d_pre.build([1,3,ai2d_pre_input_shape[1],ai2d_pre_input_shape[2]], [1,3,img_size[0],img_size[1]])
    with ScopedTiming("total",debug_mode > 0):
        ai2d_pre_builder.run(ai2d_pre_input_tensor, ai2d_pre_out)
        # set input
        kpu.set_input_tensor(0, ai2d_pre_out)
        # run kmodel
        kpu.run()
        del ai2d_pre_input_tensor
        del ai2d_pre_out

        # get output
        results = []
        for i in range(kpu.outputs_size()):
            data = kpu.get_output_tensor(i)
            result = data.to_numpy()[0]
            results.append(result)


        result_img = aicube.seg_post_process(results[0], num_classes, img_size, [ai2d_pre_input_shape[1],ai2d_pre_input_shape[2]])
        save_img = result_img[:,:,:3].copy()
        img = image.Image(ai2d_pre_input_shape[1],ai2d_pre_input_shape[2],image.RGB888,alloc=image.ALLOC_REF,data=save_img).to_rgb565()
        img.compress_for_ide()
        img.save(root_path + 'result_seg.jpg')
        del ai2d_pre
        del ai2d_pre_builder
        del kpu
        gc.collect()
    print("---------------end------------------")
    nn.shrink_memory_pool()

if __name__ == "__main__":
    nn.shrink_memory_pool()
    segmentation()

